#include "ReShadeUI.fxh"
#include "ReShade.fxh"
uniform float Sharpness < __UNIFORM_SLIDER_INT1
    ui_label = "Sharpening strength";
	ui_min = 0.0; ui_max = 5.0;
	ui_tooltip = "Sharpen the finer details according to the contrast between it neighboring pixels";
> = 1.0;
uniform float Clarity < __UNIFORM_SLIDER_INT1
    ui_label = "Clarity strength";
	ui_tooltip = "Emphasizes borders and crevices";
	ui_min = 0.0; ui_max = 5.0;
> = 1.0;
uniform uint cyan < __UNIFORM_SLIDER_INT1
	ui_min = 0; ui_max = 10;
	ui_label = "cyan";
> = 1;
uniform uint magenta < __UNIFORM_SLIDER_INT1
	ui_min = 0; ui_max = 10;
	ui_label = "magenta";
> = 2;
uniform uint yellow < __UNIFORM_SLIDER_INT1
	ui_min = 0; ui_max = 10;
	ui_label = "yellow";
> = 1;
uniform float Lumi < __UNIFORM_SLIDER_INT1
	ui_min = -1.0; ui_max = 2.0;
	ui_label = "Contrast";
> = 0;
uniform float Saturation < __UNIFORM_SLIDER_INT1
	ui_min = -1.0; ui_max = 1.0;
	ui_label = "Saturation";
> = 0.0;

float3 Adjustments(float4 vpos : SV_Position, float2 texcoord : TexCoord) : SV_Target
{

	float3 color = tex2D(ReShade::BackBuffer, texcoord).rgb;
	float gray = dot(color,1)*0.3333;
	
	//Calculating depth information
	float dep = tex2D(ReShade::DepthBuffer, texcoord).r;
	float e = tex2Doffset(ReShade::DepthBuffer, texcoord, int2(0, 1)).r;
	float f = tex2Doffset(ReShade::DepthBuffer, texcoord, int2(1, 0)).r;
	float g = tex2Doffset(ReShade::DepthBuffer, texcoord, int2(0, -1)).r;
	float h = tex2Doffset(ReShade::DepthBuffer, texcoord, int2(-1, 0)).r;
	float face = saturate(abs(dep-(e+f+g+h)*0.25)*10000)*Clarity*min((min(dep,0.001)*1000),1);
	
	//Sharpness and clarity 
	float3 a = tex2Doffset(ReShade::BackBuffer, texcoord, int2(-1, 0)).rgb;
	float3 b = tex2Doffset(ReShade::BackBuffer, texcoord, int2(1, 0)).rgb;
	float3 c = tex2Doffset(ReShade::BackBuffer, texcoord, int2(0, -1)).rgb;
	float3 d = tex2Doffset(ReShade::BackBuffer, texcoord, int2(0, 1)).rgb;
	float3 mnRGB = min(min(min(a,b),min(c,d)),color);
	float3 mxRGB = max(max(max(a,b),max(c,d)),color);
	float3 range = min(mxRGB-mnRGB,1);
	color = color*(1+Clarity*0.1)+(color-range/2-mnRGB)*(1-range)*Sharpness - range*(Clarity);
	
	//Applying depth border
	color = color*(1+face)-face;
	
	//Black levels, Saturation and Contrast
	float Loffset = Lumi+Saturation;
	color = color*(1+Lumi+(1-min(gray,1))) - (float3(cyan,magenta,yellow).rgb+Loffset)*rcp(255.0)*(1+Lumi*3);
	color = color*(1+Saturation)-gray*Saturation;
	return color;
}
technique Borderlandfy2077
{
    pass Adjustments
    {
        VertexShader = PostProcessVS;
        PixelShader = Adjustments;
    }
}